/*
    Copyright (C) HWPORT.COM
    All rights reserved.
    Author: JAEHYUK CHO <mailto:minzkn@minzkn.com>
*/

#if !defined(_ISOC99_SOURCE)
# define _ISOC99_SOURCE (1L)
#endif

#if !defined(_GNU_SOURCE)
# define _GNU_SOURCE (1L)
#endif

#include <sys/types.h>
#include <sys/inotify.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

int main(int s_argc, char **s_argv);

int main(int s_argc, char **s_argv)
{
    const char *s_monitoring_path;
    int s_fd, s_wd;

    ssize_t s_read_bytes;
    unsigned char s_buffer[32 << 10];
    struct inotify_event *s_event;
    size_t s_offset;

    s_monitoring_path = ".";
    if(s_argc >= 2) {
        s_monitoring_path = (const char *)s_argv[1];
    }

    s_fd = inotify_init();
    if(s_fd == (-1)) {
        perror("inotify_init");
	return(EXIT_FAILURE);
    }


    s_wd = inotify_add_watch(s_fd, s_monitoring_path, IN_ALL_EVENTS);
    if(s_wd == (-1)) {
        perror("inotify_add_watch");
    }
    else {
        (void)fprintf(stdout, "BEGIN MONITORING (\"%s\")...\n", s_monitoring_path);

	for(;;) {
	    s_read_bytes = read(s_fd, (void *)(&s_buffer[0]), sizeof(s_buffer));
	    if(s_read_bytes == ((ssize_t)(-1))) {
	        break;
	    }
	    if(s_read_bytes == ((ssize_t)0)) {
	        break;
	    }

            for(s_offset = (size_t)0u;(s_offset + sizeof(struct inotify_event)) <= ((size_t)s_read_bytes);) {
	        s_event = (struct inotify_event *)(&s_buffer[s_offset]);
    
                do {
		    struct {
		        uint32_t m_flag;
			const char *m_string;
		    }s_flag_table[] = {
		        {IN_ACCESS, "ACCESS"},
		        {IN_MODIFY, "MODIFY"},
		        {IN_ATTRIB, "ATTRIB"},
		        {IN_CLOSE_WRITE, "CLOSE_WRITE"},
		        {IN_CLOSE_NOWRITE, "CLOSE_NOWRITE"},
		        {IN_CLOSE, "CLOSE"},
		        {IN_MOVED_FROM, "MOVED_FROM"},
		        {IN_MOVED_TO, "MOVED_TO"},
		        {IN_MOVE, "MOVE"},
		        {IN_CREATE, "CREATE"},
		        {IN_DELETE, "DELETE"},
		        {IN_DELETE_SELF, "DELETE_SELF"},
		        {IN_MOVE_SELF, "MOVE_SELF"},
		        {IN_UNMOUNT, "UNMOUNT"},
		        {IN_Q_OVERFLOW, "Q_OVERFLOW"},
		        {IN_IGNORED, "IGNORED"},
		        {IN_ONLYDIR, "ONLYDIR"},
		        {IN_DONT_FOLLOW, "DONT_FOLLOW"},
		        {IN_EXCL_UNLINK, "EXCL_UNLINK"},
		        {IN_MASK_ADD, "MASK_ADD"},
		        {IN_ISDIR, "ISDIR"},
		        {IN_ONESHOT, "ONESHOT"},
		        {IN_CLOEXEC, "CLOEXEC"},
		        {IN_NONBLOCK, "NONBLOCK"},
		        {0, (const char *)0}
		    };
		    size_t s_flag_offset;
		    int s_flag_index;
		    unsigned char s_flag_message[ 4 << 10 ];

		    for(s_flag_index = 0, s_flag_offset = (size_t)0u;s_flag_table[s_flag_index].m_string != ((const char *)0);s_flag_index++) {
                        if((s_event->mask & s_flag_table[s_flag_index].m_flag) != s_flag_table[s_flag_index].m_flag) {
			    continue;
			}

		        s_flag_offset += (size_t)sprintf((char *)(&s_flag_message[s_flag_offset]), "%s ", s_flag_table[s_flag_index].m_string);
		    }
		    
		    (void)fprintf(stdout, "[%40s](%lu)\"%s\"\n",
		        (char *)(&s_flag_message[0]),
			(unsigned long)s_event->len,
			(s_event->len > 0) ? s_event->name : "");
		}while(0);

		s_offset += sizeof(struct inotify_event) + ((size_t)s_event->len);
	    }
	}

        (void)fprintf(stdout, "END OF MONITORING (\"%s\")...\n", s_monitoring_path);
    
        (void)inotify_rm_watch(s_fd, s_wd);
    }

    (void)close(s_fd);

    return(EXIT_SUCCESS);
}

/* vim: set expandtab: */
/* End of source */
